using System.Diagnostics.Metrics;
using Frontend.Metrics;


namespace Frontend.BackgroundServices;

public class PrometheusService : BackgroundService
{

    private readonly ILogger<PrometheusService> _logger;
    private readonly TimeSpan _interval = TimeSpan.FromSeconds(10);
    private readonly Erc20TokenMetrics _erc20TokenMetrics;
    private readonly EthMetrics _ethMetrics;
    private readonly MeterListener _meterListener;

    public PrometheusService(
        ILogger<PrometheusService> logger,
        Erc20TokenMetrics tokenMetrics,
        EthMetrics ethMetrics,
        PotatoStorageMetrics potatoStorageMetrics,
        PotatoMarketplaceMetrics potatoMarketplaceMetrics
    )
    {
        _logger = logger;
        _erc20TokenMetrics = tokenMetrics;
        _ethMetrics = ethMetrics;
        _interval = TimeSpan.FromSeconds(10);
        _meterListener = new MeterListener();
        _meterListener.InstrumentPublished = (instrument, listener) =>
        {
            Console.WriteLine($"Subscribing to {instrument.Meter.Name}\\{instrument.Name}");
            listener.EnableMeasurementEvents(instrument);
        };
        _erc20TokenMetrics.CreateMetrics();
        _ethMetrics.CreateMetrics();
        potatoStorageMetrics.CreateMetrics();
        potatoMarketplaceMetrics.CreateMetrics();
    }

    protected async override Task ExecuteAsync(CancellationToken stoppingToken)
    {
        while (!stoppingToken.IsCancellationRequested)
        {
            _logger.LogInformation("Executing at {time}", DateTime.Now);

            _meterListener.RecordObservableInstruments();
            await Task.Delay(_interval, stoppingToken);
        }
    }
}
